package com.nisum.employeeservice.controller;

import com.nisum.employeeservice.exception.ResourceNotFoundException;
import com.nisum.employeeservice.model.Employee;
import com.nisum.employeeservice.service.EmployeeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import javax.xml.ws.Response;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@CrossOrigin(origins = "*")
@RestController
@RequestMapping("/api/v1")
public class EmployeeController {

  @Autowired EmployeeService employeeService;

  @GetMapping("/employees")
  public List<Employee> getAllEmployees() {
    return employeeService.findAllEmployees();
  }

  @PostMapping("/employees")
  public Employee createEmployee(@Valid @RequestBody Employee employee) {
    return employeeService.addEmployee(employee);
  }

  @GetMapping("/employees/{id}")
  public ResponseEntity<Employee> getEmployeeById(@PathVariable(value = "id") Long employeeId) throws ResourceNotFoundException {
    Employee employee = employeeService.getEmployeeById(employeeId)
            .orElseThrow(() -> new ResourceNotFoundException("Employee by Id Not Found: " + employeeId));
    return ResponseEntity.ok().body(employee);
  }

  @PutMapping("employees/{id}")
  public ResponseEntity<Employee> updateEmployee(@PathVariable(value = "id") Long employeeId,
                                                 @Valid @RequestBody Employee employeeDetails) throws ResourceNotFoundException {
    Employee employee = employeeService.getEmployeeById(employeeId)
            .orElseThrow(() -> new ResourceNotFoundException("Employee Not Found: " + employeeId));
    Employee updatedEmployee = employeeService.editEmployee(employee, employeeDetails);
    return ResponseEntity.ok(updatedEmployee);
  }

  @DeleteMapping("employees/{id}")
  public Map<String, Boolean> deleteEmployee(@PathVariable(value = "id") Long employeeId) throws ResourceNotFoundException {
    Employee employee = employeeService.getEmployeeById(employeeId)
            .orElseThrow(() -> new ResourceNotFoundException("Employee Not Found: " + employeeId));

    employeeService.deleteEmployee(employee);
    Map<String, Boolean> response = new HashMap<>();
    response.put("deleted", Boolean.TRUE);
    return response;
  }

}
